
// Project: webCDwriter - Network CD Writing
// File: Config.cpp
// Author: Jrg Haeger, 25.05.1999

#include "Config.h"

	Config::Config()
	{
		cdrecordInfo = "cdrecord";
		fileName = getenv("CDWserverConfig");
		if (fileName == NULL)
			fileName = "/etc/CDWserver/config";
		char *configDir = strDup(fileName);
		int i;
		for (i = strlen(configDir);
			i > 0 && configDir[i - 1] != '/'; i--);
		configDir[i] = 0;
		firstError = NULL;
		mkisofsInfo = "mkisofs";
		writerInfo = "unknown";

		// values
		cdrecordDevArg = "auto";
		cdrecordPath = "cdrecord";
		debug = "";
		defaultFilesDir = "/etc/CDWserver/default_files";
		eMailOnError = NULL;
		group = "root";
		imageFile = "/var/spool/CDWserver/cdimage.iso";
		infoFile = strCat(configDir, "greeting");
		laser = "on";
		logDir = "/var/log/CDWserver";
		logoURL = NULL;
		maxMBytesInSpoolDir = 1600;
		maxMBytesPerSession = 800;
		maxOpenConnections = 10;
		maxOpenSessions = 1;
		mkisofsMArg = "auto";
		mkisofsPath = "mkisofs";
		mpg123Path = "mpg123";
		organization = "unknown";
		portNo = 10000;
		reservedMBytes = 10;
		speed = 4;
		spoolDir = "/var/spool/CDWserver";
		statusPortNo = 10001;
		user = "root";
		waitForCD = 10;
		waitForCDMsgFile = strCat(configDir, "waitForCD");

		readConfigFileLoop();
	}

	void Config::parseLine(
		const char *lineStr, char *str1, char *str2)
	{
		str1[0] = str2[0] = 0;

		if (lineStr[0] == '#')
			return;

		int i = 0;
		while (lineStr[i] == '\t'
			|| lineStr[i] == ' ')
			i++;

		int j = 0;
		for (; lineStr[i] != 0
			&& lineStr[i] != '\n'
			&& lineStr[i] != '\t'
			&& lineStr[i] != ' '
			&& lineStr[i] != '='; i++)
			str1[j++] = lineStr[i];
		str1[j] = 0;

		while (lineStr[i] == '\t'
			|| lineStr[i] == ' '
			|| lineStr[i] == '=')
			i++;

		j = 0;
		while (lineStr[i] != 0
			&& lineStr[i] != '\n')
			str2[j++] = lineStr[i++];

		// truncate trailing whitespace
		while (j > 0
			&& (str2[j - 1] == '\t'
				|| str2[j - 1] == ' '))
			j--;
		str2[j] = 0;
	}

	void Config::readConfigFile()
	{
		if (firstError != NULL)
		{
			delete firstError;
			firstError = NULL;
		}

		readConfigFileLoop();

		log.put("Config: port=%d, dev=%s, laser=%s",
			portNo,	cdrecordDevArg, laser);

		char str[4 * 1024];
		if (firstError == NULL)
		try {
			File logDir(logDir);
			if (!logDir.exists())
			{
				sprintf(str, "logDir %s does not exist",
					this->logDir);
				throw str;
			}

			File spoolDir(spoolDir);
			if (!spoolDir.exists())
			{
				sprintf(str, "spoolDir %s does not exist",
					this->spoolDir);
				throw str;
			}

			FILE *outStream = fopen(imageFile, "w");
			if (outStream == NULL)
			{
				sprintf(str, "can't create imageFile %s",
					imageFile);
				throw str;
			}
			else
				fclose(outStream);

			testMkisofs();
			testCdrecord();
			testCdrecord2();
			testCdrecord3();

			if (access(mkisofsMArg, R_OK) != 0)
			{
				sprintf(str, "don't have the permissions "
					"to read %s", mkisofsMArg);
				throw str;
			}
		}
		catch (const char *msg)
		{
			firstError = strDup(msg);
		}
	}

	void Config::readConfigFileLoop()
	{
		FILE *inStream = fopen(fileName, "r");
		if (inStream == NULL)
		{
			char str[strlen(fileName) + 1024];
			sprintf(str, "can't open configFile %s", fileName);
			firstError = strDup(str);
			return;
		}

		int lineNo = 0;
		char lineStr[1024];
		while (fgets(lineStr, sizeof lineStr, inStream) != NULL)
		{
			lineNo++;
			char str1[1024], str2[1024];
			parseLine(lineStr, str1, str2);

			int error = 0;
			if (strlen(str1) == 0 && strlen(str2) == 0)
				continue;
			else
			if (strcasecmp(str1, "cdrecordDevArg") == 0)
			{
				int bus, id, lun;
				if (sscanf(str2, "%d,%d,%d",
					&bus, &id, &lun) == 3)
					cdrecordDevArg = strDup(str2);
				else
				if (strcasecmp(str2, "auto") == 0)
					cdrecordDevArg = "auto";
				else
					error = 1;
			}
			else
			if (strcasecmp(str1, "cdrecordPath") == 0)
				cdrecordPath = strDup(str2);
			else
			if (strcasecmp(str1, "debug") == 0)
				debug = strDup(str2);
			else
			if (strcasecmp(str1, "defaultFilesDir") == 0)
				defaultFilesDir = strDup(str2);
			else
			if (strcasecmp(str1, "eMailOnError") == 0)
				eMailOnError = strDup(str2);
			else
			if (strcasecmp(str1, "group") == 0)
				group = strDup(str2);
			else
			if (strcasecmp(str1, "imageFile") == 0)
			{
				strcat(str2, ": now internal, remove line");
				error = 1;
			}
			else
			if (strcasecmp(str1, "laser") == 0)
				laser = strDup(str2);
			else
			if (strcasecmp(str1, "logDir") == 0)
				logDir = strDup(str2);
			else
			if (strcasecmp(str1, "logoURL") == 0)
				logoURL = strDup(str2);
			else
			if (strcasecmp(str1, "maxMBytesInSpoolDir") == 0)
				sscanf(str2, "%d", &maxMBytesInSpoolDir);
			else
			if (strcasecmp(str1, "maxMBytesPerSession") == 0)
				sscanf(str2, "%d", &maxMBytesPerSession);
			else
			if (strcasecmp(str1, "maxOpenConnections") == 0)
				sscanf(str2, "%d", &maxOpenConnections);
			else
			if (strcasecmp(str1, "maxOpenSessions") == 0)
			{
				if (strcasecmp(str2, "auto") == 0)
					maxOpenSessions = 0;
				else
				sscanf(str2, "%d", &maxOpenSessions);
			}
			else
			if (strcasecmp(str1, "maxStoreMBs") == 0)
			{
				strcat(str2, ": use maxMBytesInSpoolDir"
					" (see CDWserver/config.eg)");
				error = 1;
			}
			else
			if (strcasecmp(str1, "mkisofsMArg") == 0)
				mkisofsMArg = strDup(str2);
			else
			if (strcasecmp(str1, "mkisofsPath") == 0)
				mkisofsPath = strDup(str2);
			else
			if (strcasecmp(str1, "mpg123Path") == 0)
				mpg123Path = strDup(str2);
			else
			if (strcasecmp(str1, "organization") == 0)
				organization = strDup(str2);
			else
			if (strcasecmp(str1, "portNo") == 0)
				sscanf(str2, "%hd", &portNo);
			else
			if (strcasecmp(str1, "reservedMBytes") == 0)
			{
				sscanf(str2, "%d", &reservedMBytes);
				if (reservedMBytes < 10)
					reservedMBytes = 10;
			}
			else
			if (strcasecmp(str1, "speed") == 0)
				sscanf(str2, "%d", &speed);
			else
			if (strcasecmp(str1, "spoolDir") == 0)
			{
				spoolDir = strDup(str2);
				char str[1024];
				sprintf(str, "%s/cdimage.iso", spoolDir);
				imageFile = strDup(str);
			}
			else
			if (strcasecmp(str1, "statusPortNo") == 0)
				sscanf(str2, "%hd", &statusPortNo);
			else
			if (strcasecmp(str1, "user") == 0)
				user = strDup(str2);
			else
			if (strcasecmp(str1, "waitForCD") == 0)
				sscanf(str2, "%d", &waitForCD);
			else
				error = 1;

			if (error)
			{
				char str[strlen(fileName)
					+ strlen(lineStr) + 1024];
				sprintf(str, "error in %s: line %d: %s=%s",
					fileName, lineNo, str1, str2);
				if (firstError == NULL)
					firstError = strDup(str);
				log.put(str);
			}
		}

		if (maxOpenSessions == 0)
			// calculate number of possible parallel sessions
			maxOpenSessions = max(1,
				maxMBytesInSpoolDir / maxMBytesPerSession - 1);

		fclose(inStream);
	}

	void Config::testCdrecord()
	{
		Process p(getCdrecordPath());
		p.addArg("-version");
		p.start();
		const char *lineStr = p.readLine();
		if (lineStr == NULL)
			throw "can't run cdrecord";
		if (p.getResult() != 0)
			throw "error running cdrecord -version";
		char str1[strlen(lineStr) + 1], str2[strlen(lineStr) + 1];
		if (sscanf(lineStr, "%s %s", str1, str2) != 2)
			throw "can't get version of cdrecord";
		char *str3 = new char[strlen(str1) + strlen(str2) + 2];
		sprintf(str3, "%s %s", str1, str2);
		cdrecordInfo = str3;
		log.put(lineStr);
	}

void Config::testCdrecord2()
{
	Process p(getCdrecordPath());
	p.addArg("-scanbus");
	p.start();

	char CDROMs[20][20], writers[20][20];
	int numOfCDROMs = 0, numOfWriters = 0;
	while (1)
	{
		const char *lineStr = p.readLine();
		if (lineStr == NULL)
			break;
		if (strstr(lineStr, "CD-ROM") == NULL
			&& strstr(lineStr, "WORM") == NULL)
			continue;

		log.put(lineStr);
		int bus, ID, LUN;
		if (sscanf(lineStr, "%d,%d,%d", &bus, &ID, &LUN) != 3)
			continue;

		sprintf(CDROMs[numOfCDROMs++], "%d,%d,%d",
			bus, ID, LUN);
		const char *str = lineStr;
		for (int i = 0; i < 3; i++)
		{
			while (*str != '\'' && *str != 0)
				str++;
			if (*str != 0)
				str++;
		}
		char model[17];
		strncpy(model, str, 16);
		model[16] = 0;

		const char *patterns[] = {
			"CD-R", "CDR", "CR", "CW", "RW",
			"LTR", "MP-7", "PX-R", "PX-W", NULL };
		if (strstr(model, "CDROM") == NULL
			&& strstr(model, "CD-ROM") == NULL)
		{
			const char **p = patterns;
			while (*p != NULL && strstr(model, *p) == NULL)
				p++;
			if (*p != NULL)
				sprintf(writers[numOfWriters++], "%d,%d,%d",
					bus, ID, LUN);
		}
	}

	if (p.getResult() != 0)
		throw "error running cdrecord -scanbus";

	log.put("CDROMs = %d, writers = %d", numOfCDROMs, numOfWriters);
	if (strcasecmp(cdrecordDevArg, "auto") == 0)
		if (numOfWriters == 0)
			if (numOfCDROMs == 0)
				throw "Cannot autodetect cdrecordDevArg";
			else
				cdrecordDevArg = strDup(CDROMs[0]);
		else
			cdrecordDevArg = strDup(writers[0]);

	if (strcasecmp(mkisofsMArg, "auto") == 0)
	{
		int i = 0;
		for (i = 0; i < numOfCDROMs; i++)
			if (strcmp(CDROMs[i], cdrecordDevArg) == 0)
				break;

		if (i < numOfCDROMs)
		{
			char str[1024];
			sprintf(str, "/dev/scd%d", i);
			mkisofsMArg = strDup(str);
		}
		else
			throw "Cannot autodetect mkisofsMArg";
	}

	log.put("cdrecordDevArg = %s mkisofsMArg = %s",
		cdrecordDevArg, mkisofsMArg);
}

	void Config::testCdrecord3()
	{
		Process p(getCdrecordPath());
		char str[4*1024];
		sprintf(str, "dev=%s", getCdrecordDevArg());
		p.addArg(str);
		p.addArg("-inq");
		p.start();

		int errorCode = 0;
		str[0] = 0;
		while (1)
		{
			const char *lineStr = p.readLine();
			if (lineStr == NULL)
				break;

			char a[strlen(lineStr) + 1], b[strlen(lineStr) + 1];
			if (sscanf(lineStr, "%s : '%s", a, b) == 2)
			if (strcmp(a, "Vendor_info") == 0
				|| strcmp(a, "Identifikation") == 0
				|| strcmp(a, "Revision") == 0)
			{
				strcat(str, b);
				strcat(str, " ");
			}
		}

		if (p.getResult() != 0)
		{
			sprintf(str, "error running cdrecord dev=%s -inq",
				getCdrecordDevArg());
			throw str;
		}

		if (strlen(str) >= 2 && str[strlen(str) - 2] == '\'')
			str[strlen(str) - 2] = 0;
		writerInfo = strDup(str);
	}

	void Config::testMkisofs()
	{
		Process p(getMkisofsPath());
		p.addArg("-version");
		p.start();
		const char *lineStr = p.readLine();
		if (lineStr == NULL)
			throw "can't run mkisofs";
		if (p.getResult() != 0)
			throw "error running mkisofs -version";
		char str1[strlen(lineStr) + 1], str2[strlen(lineStr) + 1];
		if (sscanf(lineStr, "%s %s", str1, str2) != 2)
			throw "can't get version of mkisofs";
		char *str3 = new char[strlen(str1) + strlen(str2) + 2];
		sprintf(str3, "%s %s", str1, str2);
		mkisofsInfo = str3;
		log.put(lineStr);
	}

