
#ifndef CONFIG_H
#define CONFIG_H

// Project: webCDwriter - Network CD Writing
// File: Config.h
// Author: Jrg Haeger, 25.05.1999

#include <stdio.h>
#include <string.h>

#include "File.h"
#include "Process.h"
#include "Support.h"
#include "Log.h"

/**
 * delivery of the values of the configuration file
 */
class Config
{
	const char *cdrecordInfo;
	const char *fileName;
	const char *firstError;
	const char *mkisofsInfo;
	const char *writerInfo;

	// values
	const char *cdrecordDevArg;
	const char *cdrecordPath;
	const char *debug;
	const char *defaultFilesDir;
	const char *eMailOnError;
	const char *group;
	const char *imageFile;
	const char *infoFile;
	const char *laser;
	const char *logDir;
	const char *logoURL;
	int maxMBytesInSpoolDir;
	int maxMBytesPerSession;
	int maxOpenConnections;
	int maxOpenSessions;
	const char *mkisofsMArg;
	const char *mkisofsPath;
	const char *mpg123Path;
	const char *organization;
	unsigned short int portNo;
	int reservedMBytes;
	int speed;
	const char *spoolDir;
	unsigned short int statusPortNo;
	const char *user;
	int waitForCD;
	const char *waitForCDMsgFile;

public:
	Config();

	const char *getCdrecordDevArg()
	{
		return cdrecordDevArg;
	}
	const char *getCdrecordInfo()
	{
		return cdrecordInfo;
	}
	const char *getCdrecordPath()
	{
		return cdrecordPath;
	}
	const char *getDebug()
	{
		return debug;
	}
	const char *getDefaultFilesDir()
	{
		return defaultFilesDir;
	}
	const char *getEMailOnError()
	{
		return eMailOnError;
	}
	const char *getFirstError()
	{
		return firstError;
	}
	const char *getGroup()
	{
		return group;
	}
	const char *getImageFile()
	{
		return imageFile;
	}
	const char *getInfoFile()
	{
		return infoFile;
	}
	int getLaserOn()
	{
		return strcasecmp(laser, "on") == 0;
	}
	const char *getLogDir()
	{
		return logDir;
	}
	const char *getLogoURL()
	{
		return logoURL;
	}
	int getMaxMBytesInSpoolDir()
	{
		return maxMBytesInSpoolDir;
	}
	int getMaxMBytesPerSession()
	{
		return maxMBytesPerSession;
	}
	int getMaxOpenConnections()
	{
		return maxOpenConnections;
	}
	int getMaxOpenSessions()
	{
		return maxOpenSessions;
	}
	int getMaxStoreMBs()
	{
		return maxMBytesInSpoolDir - maxMBytesPerSession;
	}
	const char *getMkisofsInfo()
	{
		return mkisofsInfo;
	}
	const char *getMkisofsMArg()
	{
		return mkisofsMArg;
	}
	const char *getMkisofsPath()
	{
		return mkisofsPath;
	}
	const char *getMpg123Path()
	{
		return mpg123Path;
	}
	const char *getOrganization()
	{
		return organization;
	}
	unsigned short int getPortNo()
	{
		return portNo;
	}
	int getReservedMBytes()
	{
		return reservedMBytes;
	}
	int getSpeed()
	{
		return speed;
	}
	const char *getSpoolDir()
	{
		return spoolDir;
	}
	unsigned short int getStatusPortNo()
	{
		return statusPortNo;
	}
	const char *getUser()
	{
		return user;
	}
	int getWaitForCD()
	{
		return waitForCD;
	}
	const char *getWaitForCDMsgFile()
	{
		return waitForCDMsgFile;
	}
	const char *getWriterInfo()
	{
		return writerInfo;
	}

	static void parseLine(
		const char *lineStr, char *str1, char *str2);
	void readConfigFile();
	void readConfigFileLoop();
	void testCdrecord();
	void testCdrecord2();
	void testCdrecord3();
	void testMkisofs();
};

extern Config config;

#endif

