
#ifndef FILE_H
#define FILE_H

#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include "Support.h"

class File
{
	char *pathName;

public:
	File(const char *pathName)
	{
		File::pathName = strDup(pathName);
	}

	File(const char *parent, const char *child)
	{
		pathName = new char[
			strlen(parent) + 1 + strlen(child) + 1];
		sprintf(pathName, "%s/%s", parent, child);
	}

	File(const File &parent, const char *child)
	{
		pathName = new char[
			strlen(parent.getName()) + 1 + strlen(child) + 1];
		sprintf(pathName, "%s/%s", parent.getName(), child);
	}

	File(const File *parent, const char *child)
	{
		if (parent == NULL)
		{
			pathName = strDup(child);
			return;
		}

		pathName = new char[
			strlen(parent->getName()) + 1 + strlen(child) + 1];
		sprintf(pathName, "%s/%s", parent->getName(), child);
	}

	~File()
	{
		delete pathName;
	}

	int exists() const
	{
		struct stat statBuf;
		return stat(pathName, &statBuf) == 0;
	}

	const char *getName() const
	{
		return pathName;
	}

	int isDirectory() const
	{
		struct stat statBuf;
		if (stat(pathName, &statBuf) != 0)
			return 0;
		return S_ISDIR(statBuf.st_mode);
	}

	void mkdirs() const
	{
//		char str[strlen(pathName) + 2];
//		sprintf(str, "%s/", pathName);
//		makeDirs(str);
		makeDirs(pathName);
	}

	void remove()
	{
		remove(pathName);
	}
private:
	static void remove(const char *pathName)
	{
		struct stat statBuf;
		if (lstat(pathName, &statBuf) != 0)
			return;

		if (!S_ISDIR(statBuf.st_mode))
		{
			unlink(pathName);
			return;
		}

		DIR *dir = opendir(pathName);
		if (dir == NULL)
			return;

		struct dirent *dirEntry;
		while ((dirEntry = readdir(dir)) != NULL)
		{
			if (strcmp(dirEntry->d_name, ".") == 0
				|| strcmp(dirEntry->d_name, "..") == 0)
				continue;

			char tmpPathName[strlen(pathName) + 1
				+ strlen(dirEntry->d_name) + 1];
			sprintf(tmpPathName, "%s/%s",
				pathName, dirEntry->d_name);
			remove(tmpPathName);
		}

		closedir(dir);
		rmdir(pathName);
	}
};

#endif

