
#include <stdarg.h>
#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>

#include "Version.h"
#include "Config.h"
#include "Log.h"

Log::Log()
{
	active = 0;
}

void Log::debug(const char *tag, const char *format, ...)
{
	if (strstr(config.getDebug(), tag) != NULL)
	{
		char str[1024];
		va_list ap;
		va_start(ap, format);
		vsnprintf(str, sizeof str, format, ap);
		va_end(ap);
		putv("%s: %s", tag, str);
	}
}

void Log::put(const char *format, ...)
{
	if (!active)
		return;

	char str[1024];
	va_list ap;
	va_start(ap, format);
	vsnprintf(str, sizeof str, format, ap);
	va_end(ap);

	int l = strlen(str);
	if (l > 0 && str[l-1] == '\n')
		str[l-1] = 0;

	static char oldStr[sizeof str];
	if (strcmp(str, oldStr) != 0)
	{
		fprintf(file, "%d: %s\n", getpid(), str);
		if (file != file2)
			fprintf(file2, "%d: %s\n", getpid(), str);
	}
	strcpy(oldStr, str);

	fflush(file);
	fflush(file2);
}

void Log::putv(const char *format, ...)
{
	if (!active)
		return;

	char str[1024];
	va_list ap;
	va_start(ap, format);
	vsnprintf(str, sizeof str, format, ap);
	va_end(ap);

	int l = strlen(str);
	if (l > 0 && str[l-1] == '\n')
		str[l-1] = 0;

	static char oldStr[sizeof str];
	if (strcmp(str, oldStr) != 0)
		fprintf(file2, "%d: %s\n", getpid(), str);
	strcpy(oldStr, str);

	fflush(file2);
}

void Log::start(int putvToStdOut)
{
	char fileName[4*1024];
	sprintf(fileName, "%s/%s.log",
		config.getLogDir(), version);
	file = fopen(fileName, "a");
	if (file == NULL)
		file = stdout;

	if (putvToStdOut)
		file2 = stdout;
	else
	{
		sprintf(fileName, "%s/%s.vlog",
			config.getLogDir(), version);
		file2 = fopen(fileName, "a");
		if (file2 == NULL)
			file2 = stdout;
	}

	active = 1;

	time_t t;
	time(&t);
	log.put("*** %s %s, %s", product, version, ctime(&t));
	log.put("uid = %d, euid = %d, gid = %d, egid = %d",
		getuid(), geteuid(), getgid(), getegid());

	fflush(file);
	fflush(file2);
}

