
#ifndef STORE_H
#define STORE_H

#include <pthread.h>

class File;
class Server;
class Session;

class Store
{
	pthread_mutex_t mutex;
	File *spoolDir;
	int maxOpenSessions, sessionsNum, sessionsTotal;
	Session **sessions;
	Queue *queue;
	int
		fsBlockSize, reservedBlocks,
		maxSizeInKBytes, sizeInKBytes,
		KBytesAnnounced;
	dev_t imageFileDevice, spoolDirDevice;

public:
	Store();
	~Store();
	void announceKBytes(int n);
	void cancelKBytes(int n);
	int getSizeMBs() { return sizeInKBytes / 1024; }
	int getSessionsNum() { return sessionsNum; }
	int getSessionsTotal() { return sessionsTotal; }
	void registerKBytes(long n);
	File *registerSession(Session *session);
	void removeDir(const char *dirName);
	void unregister(Session *session);

private:
	void deleteOldestSession();
	void freeSpace();
	int getReservedBlocksForImageFile();
	int isSessionIDInUse(const char *sessionID);
	void register2(Session *session);
	void register3(Session *session);
	int sizeToBlocks(long size);
};

extern Store *store;

#endif

